@@ -1,5 +1,10 @@
+"""
+spangle - A small and flexible ASGI application framework for modern web.
+"""
+
 from .api import Api
 from .blueprint import Blueprint
+from .component import use_api, use_component
 from .error_handler import ErrorHandler
 from .models import Connection, Request, Response
 from .parser import UploadedFile
@@ -7,6 +12,8 @@ from .parser import UploadedFile
 __all__ = [
     "Api",
     "Blueprint",
+    "use_api",
+    "use_component",
     "ErrorHandler",
     "UploadedFile",
     "Request",
