@@ -3,26 +3,18 @@ HTTP Request & Response.
 """
 
 
+from collections.abc import AsyncGenerator, Awaitable, Callable
 from http import HTTPStatus
 from http.cookies import SimpleCookie
-from typing import (
-    Any,
-    AsyncGenerator,
-    Awaitable,
-    Callable,
-    Dict,
-    List,
-    Optional,
-    Tuple,
-    Type,
-    Union,
-)
+from typing import Any, Optional, Union
 from urllib.parse import parse_qsl, unquote_plus
 
 import addict
 import chardet
 import jinja2
 from multidict import CIMultiDict, CIMultiDictProxy, MultiDict, MultiDictProxy
+from spangle.exceptions import NotFoundError, TooLargeRequestError
+from spangle.parser import _parse_body
 from starlette.requests import URL, Address
 from starlette.requests import Request as StarletteRequest
 from starlette.responses import JSONResponse, RedirectResponse
@@ -30,9 +22,6 @@ from starlette.responses import Response as StarletteResponse
 from starlette.responses import StreamingResponse
 from starlette.types import Receive, Scope, Send
 
-from spangle.exceptions import NotFoundError, TooLargeRequestError
-from spangle.parser import _parse_body
-
 
 class _Accept:
     """
@@ -102,7 +91,7 @@ class Request:
     )
 
     _request: StarletteRequest
-    _accepts: Optional[List[_Accept]]
+    _accepts: Optional[list[_Accept]]
     _content: Optional[bytes]
     _mimetype: Optional[str]
     _full_url: Optional[str]
@@ -134,9 +123,9 @@ class Request:
     async def content(self) -> bytes:
         """
         (`bytes`): The request body, as bytes. Must be awaited.
-        
+
         **Raises**
-        
+
         * `spangle.exceptions.TooLargeRequestError` : when request body is too large.
 
         """
@@ -156,8 +145,8 @@ class Request:
         return self._content
 
     @property
-    def cookies(self) -> Dict[str, str]:
-        """(`Dict[str, str]`): The cookies sent in the request, as a dictionary."""
+    def cookies(self) -> dict[str, str]:
+        """(`dict[str, str]`): The cookies sent in the request, as a dictionary."""
         return self._request.cookies
 
     @property
@@ -227,7 +216,7 @@ class Request:
         else:
             return 1
 
-    def accept(self, content_type: str) -> Optional[Tuple[str, float]]:
+    def accept(self, content_type: str) -> Optional[tuple[str, float]]:
         """
         Test given type is acceptable or not.
 
@@ -237,13 +226,13 @@ class Request:
 
         **Returns**
 
-        * Optional[`Tuple[str, float]`]: The first accepted type and its priority in
+        * Optional[`tuple[str, float]`]: The first accepted type and its priority in
             the range: `0.0<=q<=1.0` , or `None` .
 
         """
 
         if self._accepts is None:
-            raw: List[str] = self.headers.getall("Accept", [])
+            raw: list[str] = self.headers.getall("Accept", [])
             a_list = []
             for i in raw:
                 a_values = i.replace(" ", "").split(",")
@@ -258,7 +247,7 @@ class Request:
         return None
 
     @property
-    async def apparent_encoding(self) -> Dict[str, Union[str, float]]:
+    async def apparent_encoding(self) -> dict[str, Union[str, float]]:
         """
         (`Dict[str, Union[str, float]]`): Guess the content encoding, provided by the
             `chardet` library. Must be awaited.
@@ -355,15 +344,15 @@ class Response:
     )
 
     _jinja: Optional[jinja2.Environment]
-    _redirect_to: Optional[Tuple[str, Optional[str]]]
+    _redirect_to: Optional[tuple[str, Optional[str]]]
     _url_for: Optional[Callable]
-    _starlette_resp: Type[StarletteResponse]
+    _starlette_resp: type[StarletteResponse]
     _body: Any
     _text: Optional[str]
     _content: Optional[bytes]
     _json: Union[addict.Dict, list, None]
 
-    headers: CIMultiDict
+    headers: CIMultiDict[str]
     cookies: SimpleCookie
     status_code: int
     streaming: Optional[AsyncGenerator]
@@ -413,7 +402,7 @@ class Response:
 
     def set_status(self, status: int) -> "Response":
         """
-        Set HTTP status code. Chainable
+        Set HTTP status code.
 
         **Args**
 
@@ -443,7 +432,7 @@ class Response:
     ) -> "Response":
 
         """
-        Set cookie value to given key with params. Chainable
+        Set cookie value to given key with params.
 
         **Args**
 
@@ -491,7 +480,7 @@ class Response:
         self, key: str, path: str = "/", domain: str = None
     ) -> "Response":
         """
-        Remove cookie value from client. Chainable
+        Remove cookie value from client.
 
         **Args**
 
@@ -551,7 +540,6 @@ class Response:
     def add_header(self, key: str, value: str) -> "Response":
         """
         Append new header. To overwrite, use `spangle.models.http.Response.set_header` .
-            Chainable
 
         **Args**
 
@@ -568,7 +556,7 @@ class Response:
 
     def set_header(self, key: str, value: str) -> "Response":
         """
-        Set HTTP header value to given key. It overwrites value if exists. Chainable
+        Set HTTP header value to given key. It overwrites value if exists.
 
         **Args**
 
@@ -585,7 +573,7 @@ class Response:
 
     def set_text(self, text: str, content_type="text/plain") -> "Response":
         """
-        Set given text to response body with content type. Chainable
+        Set given text to response body with content type.
 
         **Args**
 
@@ -610,7 +598,7 @@ class Response:
         self, content: bytes, content_type="application/octet-stream"
     ) -> "Response":
         """
-        Set bytes to response body with content type. Chainable
+        Set bytes to response body with content type.
 
         **Args**
 
@@ -632,7 +620,7 @@ class Response:
         self, template_name: str, content_type="text/html", **params
     ) -> "Response":
         """
-        Load `jinja2` template, render, set headers & text. Chainable
+        Load `jinja2` template, render, set headers & text.
 
         **Args**
 
@@ -675,7 +663,7 @@ class Response:
         query_string: Optional[str] = None,
     ) -> "Response":
         """
-        Set redirect view/location. Positional args are not allowed. Chainable
+        Set redirect view/location. Positional args are not allowed.
 
         If both `view` and `url` are set, `url` is ignored.
 
@@ -693,7 +681,7 @@ class Response:
         """
 
         if bool(view) is bool(url):
-            raise TypeError("Set one location; view-class or url.")
+            raise TypeError("Set only one location; view-class or url.")
         if not (300 <= status < 400):
             raise ValueError("Set correct status.")
 
@@ -704,6 +692,8 @@ class Response:
             redirect_to = self._url_for(view, params)
         elif url:
             redirect_to = url
+        else:
+            raise TypeError("Set only one location; view-class or url.")
 
         self._redirect_to = redirect_to, query_string
         return self
