@@ -1,32 +1,33 @@
 """
 Test client for ASGI app without ASGI server.
 """
-import asyncio
+from __future__ import annotations
+
+from collections.abc import Mapping
 from http import HTTPStatus
 from http.cookies import SimpleCookie
 from json import JSONDecodeError
-from typing import AnyStr, List, Mapping, Optional, Tuple, Type, TypeVar, Union
+from typing import AnyStr, Optional, TypeVar, Union
 from urllib.parse import quote_plus
 
 import addict
 from asgiref.testing import ApplicationCommunicator
 from asgiref.timeout import timeout as timeout_ctx
-from httpx import ASGIDispatch, AsyncClient, Response
+from httpx import ASGITransport, AsyncClient, Response
 from multidict import CIMultiDict
 from starlette.types import ASGIApp
 from urllib3.filepost import RequestField, encode_multipart_formdata
 
+from ._utils import _AppRef
+
 T = TypeVar("T")
-Headers = Union[Mapping, List[Tuple[str, str]]]
-Params = Union[Mapping, List[Tuple[str, str]]]
+Headers = Union[Mapping[str, str], list[tuple[str, str]]]
+Params = Union[Mapping[str, str], list[tuple[str, str]]]
 
 
 __all__ = [
-    "HttpTestResponse",
     "AsyncWebsocketClient",
-    "WebsocketClient",
     "AsyncHttpTestClient",
-    "HttpTestClient",
 ]
 
 
@@ -42,7 +43,7 @@ class HttpTestResponse:
 
     status_code: Union[HTTPStatus, int]
     _resp: Response
-    _headers: Optional[CIMultiDict] = None
+    _headers: Optional[CIMultiDict[str]] = None
     _json: Optional[addict.Dict] = None
 
     def __init__(self, resp: Response):
@@ -57,7 +58,7 @@ class HttpTestResponse:
     def headers(self) -> CIMultiDict:
         """(`CIMultiDict`): Response header, as `dict` ."""
         if self._headers is None:
-            self._headers = CIMultiDict(self._resp.headers.items())
+            self._headers = CIMultiDict(self._resp.headers.multi_items())
         return self._headers
 
     @property
@@ -90,23 +91,23 @@ class HttpTestResponse:
 
 
 class _BaseWebSocket:
-    _app: ASGIApp
+    _app_ref: _AppRef
     host: str
     path: str
     headers: CIMultiDict
     params: Params
-    timeout: Optional[int]
+    timeout: Optional[float]
 
     def __init__(
         self,
-        http: "_BaseClient",
+        http: _BaseClient,
         path: str = "",
-        headers: CIMultiDict = None,
-        params: Params = None,
-        cookies: Mapping = None,
-        timeout: int = None,
+        headers: Optional[CIMultiDict] = None,
+        params: Optional[Params] = None,
+        cookies: Optional[Mapping] = None,
+        timeout: Optional[float] = None,
     ):
-        self._app = http._app
+        self._app_ref = _AppRef(app=http._app_ref["app"])
         self.host = http.host
         self._client = http._client
         self.path = path
@@ -121,7 +122,7 @@ class _BaseWebSocket:
     async def _connect(self, path: str = None):
         self.path = path or self.path
         params = self.params or []
-        if hasattr(params, "items"):
+        if isinstance(params, Mapping):
             qsl = [f"{quote_plus(k)}={quote_plus(v)}" for k, v in params.items()]
         else:
             qsl = [f"{quote_plus(k)}={quote_plus(v)}" for k, v in params]
@@ -147,7 +148,7 @@ class _BaseWebSocket:
             ],
         }
 
-        self._connection = ApplicationCommunicator(self._app, scope)
+        self._connection = ApplicationCommunicator(self._app_ref["app"], scope)
         await self._connection.send_input({"type": "websocket.connect"})
         message = await (self._connection.receive_output(self.timeout))
         if message["type"] != "websocket.accept":
@@ -159,12 +160,14 @@ class _BaseWebSocket:
         await self._connection.receive_nothing()
         del self._connection
 
-    async def _receive(self, mode: Type[AnyStr]) -> AnyStr:
+    async def _receive(self, mode: type[AnyStr]) -> AnyStr:
         message = await self._connection.receive_output(self.timeout)
         if mode is str:
             type_key = "text"
         elif mode is bytes:
             type_key = "bytes"
+        else:
+            raise TypeError(f"`str` or `bytes` are allowed, not `{mode}` .")
         result = message.get(type_key, None)
         if result is None:
             if message["type"] == "websocket.close":
@@ -178,6 +181,10 @@ class _BaseWebSocket:
             type_key = "text"
         elif isinstance(data, bytes):
             type_key = "bytes"
+        else:
+            raise TypeError(
+                f"use `str` or `bytes`. data type `{type(data)}` is not acceptable."
+            )
         message[type_key] = data
         await self._connection.send_input(message)
 
@@ -204,7 +211,7 @@ class AsyncWebsocketClient(_BaseWebSocket):
 
     """
 
-    _app: ASGIApp
+    _app_ref: _AppRef
     host: str
     path: str
     headers: CIMultiDict
@@ -247,13 +254,13 @@ class AsyncWebsocketClient(_BaseWebSocket):
         """
         return await self._close(status_code)
 
-    async def receive(self, mode: Type[AnyStr]) -> AnyStr:
+    async def receive(self, mode: type[AnyStr]) -> AnyStr:
         """
         Receive data from the endpoint.
 
         **Args**
 
-        * mode(`Type[AnyStr]`): Receiving type, `str` or `bytes` .
+        * mode(`type[AnyStr]`): Receiving type, `str` or `bytes` .
 
         **Returns**
 
@@ -274,120 +281,31 @@ class AsyncWebsocketClient(_BaseWebSocket):
         return await self._send(data)
 
 
-class WebsocketClient(_BaseWebSocket):
-    """
-    WebSocket test client. It is expected to be called from
-        `spangle.testing.HttpTestClient` .
-
-    **Attributes**
-
-    * host(`str`): Dummy domain.
-    * path(`str`): WebSocket endpoint.
-    * headers(`CIMultiDict`): Headers used to connect.
-    * params(`Params`): Parsed querystrings.
-    * timeout(`Optional[int]`): How long test client waits for.
-
-    """
-
-    _app: ASGIApp
-    host: str
-    path: str
-    headers: CIMultiDict
-    params: Params
-    timeout: Optional[int]
-    _loop: asyncio.AbstractEventLoop
-
-    def __init__(
-        self,
-        http: "HttpTestClient",
-        path: str = "",
-        headers: CIMultiDict = None,
-        params: Params = None,
-        cookies: Mapping = None,
-        timeout: int = None,
-    ):
-        """
-        Do not use manually.
-        """
-        super().__init__(http, path, headers, params, cookies, timeout)
-        self._loop = asyncio.get_event_loop()
-
-    def connect(self, path: str = None):
-        """
-        Emulate WebSocket Connection.
-
-        **Args**
-
-        * path(`Optional[str]`): Overwrite `self.path` .
-
-        """
-        return self._loop.run_until_complete(self._connect(path))
-
-    def close(self, status_code=1000):
-        """
-        Close the connection.
-
-        **Args**
-
-        * status_code(`int`): WebSocket status code.
-
-        """
-        return self._loop.run_until_complete(self._close(status_code))
-
-    def receive(self, mode: Type[AnyStr]) -> AnyStr:
-        """
-        Receive data from the endpoint.
-
-        **Args**
-
-        * mode(`Type[AnyStr]`): Receiving type, `str` or `bytes` .
-
-        **Returns**
-
-        * `AnyStr`: Data with specified type.
-
-        """
-        return self._loop.run_until_complete(self._receive(mode))
-
-    def send(self, data: AnyStr):
-        """
-        Send data to the endpoint.
-
-        **Args**
-
-        * data(`AnyStr`): Data sent to the endpoint, must be `str` or `bytes` .
-
-        """
-        return self._loop.run_until_complete(self._send(data))
-
-    def __enter__(self) -> "WebsocketClient":
-        return self._loop.run_until_complete(self.__aenter__())
-
-    def __exit__(self, exc_type, exc_value, traceback) -> None:
-        return self._loop.run_until_complete(
-            self.__aexit__(exc_type, exc_value, traceback)
-        )
-
-
 class _BaseClient:
+    _app_ref: _AppRef
+    _transport: ASGITransport
+    host: str
+    _client: AsyncClient
+    timeout: Union[float, None]
+
     def __init__(
         self,
         app: ASGIApp,
-        timeout: Union[int, float, None] = 1,
+        timeout: Union[float, None] = 1,
         host="www.example.com",
         client=("127.0.0.1", 123),
     ):
-        self._app = app
-        self._dispatch = ASGIDispatch(app, client=client)
+        self._app_ref = _AppRef(app=app)
+        self._transport = ASGITransport(app, client=client)
         self.host = host
         self._client = AsyncClient(
-            dispatch=self._dispatch, base_url=f"http://{host}", timeout=timeout
+            transport=self._transport, base_url=f"http://{host}", timeout=timeout
         )
         self.timeout = timeout
 
     async def __aenter__(self):
         app = ApplicationCommunicator(
-            self._app,
+            self._app_ref["app"],
             {"type": "lifespan", "asgi": {"version": "3.0", "spec_version": "2.0"}},
         )
         await app.send_input({"type": "lifespan.startup"})
@@ -399,7 +317,7 @@ class _BaseClient:
 
     async def __aexit__(self, exc_type, exc, tb):
         app = ApplicationCommunicator(
-            self._app,
+            self._app_ref["app"],
             {"type": "lifespan", "asgi": {"version": "3.0", "spec_version": "2.0"}},
         )
         await app.send_input({"type": "lifespan.shutdown"})
@@ -419,20 +337,22 @@ class _BaseClient:
         files: Optional[Mapping] = None,
         form: Optional[Mapping] = None,
         content: Optional[bytes] = None,
-        timeout: Optional[int] = None,
+        timeout: Optional[float] = None,
         allow_redirects=True,
     ) -> HttpTestResponse:
         # TODO: max_redirects?
         if isinstance(headers, list):
-            headers = CIMultiDict(headers)
+            ci_headers = CIMultiDict(headers)
         elif headers is None:
-            headers = CIMultiDict()
+            ci_headers = CIMultiDict()
+        else:
+            ci_headers = CIMultiDict(headers)
         if files:
             file_list = [
                 RequestField.from_tuples(key, value) for key, value in files.items()
             ]
             data, content_type = encode_multipart_formdata(file_list)
-            headers["content-type"] = content_type
+            ci_headers["content-type"] = content_type
         elif form:
             data = form
         else:
@@ -441,7 +361,7 @@ class _BaseClient:
         if cookies is not None:
             self._client.cookies = {}
         if timeout is not None:
-            _headers = [(k, v) for k, v in headers.items()]
+            _headers = [(k, v) for k, v in ci_headers.items()]
             async with timeout_ctx(timeout):
                 response = await self._client.request(
                     method.upper(),
@@ -469,7 +389,7 @@ class _BaseClient:
                     )
 
         else:
-            _headers = [(k, v) for k, v in headers.items()]
+            _headers = [(k, v) for k, v in ci_headers.items()]
             response = await self._client.request(
                 method.upper(),
                 path,
@@ -516,7 +436,7 @@ class AsyncHttpTestClient(_BaseClient):
         * app (`ASGIApp`): Application instance.
         * timeout (`Optional[int]`): Timeout seconds.
         * host (`str`): Temporary host name.
-        * client(`Tuple[str, int]`): Client address.
+        * client(`tuple[str, int]`): Client address.
         """
         super().__init__(app, timeout, host, client)
 
@@ -574,7 +494,7 @@ class AsyncHttpTestClient(_BaseClient):
     def ws_connect(
         self,
         path: str,
-        subprotocols: List[str] = None,
+        subprotocols: list[str] = None,
         params: Params = None,
         headers: Headers = None,
         cookies: Mapping = None,
@@ -730,243 +650,3 @@ class AsyncHttpTestClient(_BaseClient):
             timeout=timeout,
             allow_redirects=allow_redirects,
         )
-
-
-class HttpTestClient(_BaseClient):
-    """
-    Mock HTTP client without running server. Lifespan-event is supported by `with`
-        statement.
-    """
-
-    def __init__(
-        self,
-        app: ASGIApp,
-        timeout: Union[int, float, None] = 1,
-        host="www.example.com",
-        client=("127.0.0.1", 123),
-    ):
-        """
-        **Args**
-
-        * app (`ASGIApp`): Application instance.
-        * timeout (`Optional[int]`): Timeout seconds.
-        * host (`str`): Temporary host name.
-        * client(`Tuple[str, int]`): Client address.
-
-        """
-        super().__init__(app, timeout, host, client)
-        self._loop = asyncio.get_event_loop()
-
-    def __enter__(self):
-        return self._loop.run_until_complete(self.__aenter__())
-
-    def __exit__(self, exc_type, exc, tb):
-        return self._loop.run_until_complete(self.__aexit__(exc_type, exc, tb))
-
-    def request(
-        self,
-        method: str,
-        path: str,
-        params: Params = None,
-        headers: Headers = None,
-        cookies: Mapping = None,
-        json: Mapping = None,
-        files: Mapping = None,
-        form: Mapping = None,
-        content: bytes = None,
-        timeout: int = None,
-        allow_redirects=True,
-    ) -> HttpTestResponse:
-        """
-        Send request to `app`.
-
-        **Args**
-
-        * method (`str`): HTTP request method.
-        * path (`str`): Requesting location.
-        * params (`Params`): Querystring as `dict` or `list` of `(name, value)`.
-        * headers (`Headers`): HTTP headers.
-        * cookies (`Mapping`): Sending HTTP cookies.
-        * json (`Mapping`): Request body as json.
-        * files (`Mapping`): Multipart form.
-        * form (`Mapping`): URL encoded form.
-        * content (`bytes`): Request body as bytes.
-        * timeout (`int`): Wait limits.
-        * allow_redirects (`bool`): If `False` , a client gets `30X` response
-            instead of redirection.
-
-        **Returns**
-
-        * `spangle.testing.HttpTestResponse`
-
-        """
-        return self._loop.run_until_complete(
-            self._request(
-                method,
-                path,
-                params=params,
-                headers=headers,
-                cookies=cookies,
-                json=json,
-                files=files,
-                form=form,
-                content=content,
-                timeout=timeout,
-                allow_redirects=allow_redirects,
-            )
-        )
-
-    def ws_connect(
-        self,
-        path: str,
-        subprotocols: List[str] = None,
-        params: Params = None,
-        headers: Headers = None,
-        cookies: Mapping = None,
-        timeout: int = None,
-    ) -> WebsocketClient:
-        """
-        Create WebSocket Connection.
-        """
-        headers = CIMultiDict(headers or {})
-        headers.setdefault("connection", "upgrade")
-        headers.setdefault("sec-websocket-key", "testserver==")
-        headers.setdefault("sec-websocket-version", "13")
-        if subprotocols is not None:
-            headers.setdefault("sec-websocket-protocol", ", ".join(subprotocols))
-
-        return WebsocketClient(self, path, headers, params, cookies, timeout)
-
-    def get(
-        self,
-        path: str,
-        params: Params = None,
-        headers: Headers = None,
-        cookies: Mapping = None,
-        timeout: int = None,
-        allow_redirects=True,
-    ) -> HttpTestResponse:
-        """
-        Send `GET` request to `app` . See `spangle.testing.HttpTestClient.request` .
-        """
-        return self.request(
-            "get",
-            path,
-            params=params,
-            headers=headers,
-            cookies=cookies,
-            timeout=timeout,
-            allow_redirects=allow_redirects,
-        )
-
-    def post(
-        self,
-        path: str,
-        params: Params = None,
-        headers: Headers = None,
-        cookies: Mapping = None,
-        json: Mapping = None,
-        files: Mapping = None,
-        form: Mapping = None,
-        content: bytes = None,
-        timeout: int = None,
-        allow_redirects=True,
-    ) -> HttpTestResponse:
-        """
-        Send `POST` request to `app` . See `spangle.testing.HttpTestClient.request` .
-        """
-        return self.request(
-            "post",
-            path,
-            params=params,
-            headers=headers,
-            cookies=cookies,
-            json=json,
-            files=files,
-            form=form,
-            content=content,
-            timeout=timeout,
-            allow_redirects=allow_redirects,
-        )
-
-    def put(
-        self,
-        path: str,
-        params: Params = None,
-        headers: Headers = None,
-        cookies: Mapping = None,
-        json: Mapping = None,
-        files: Mapping = None,
-        form: Mapping = None,
-        content: bytes = None,
-        timeout: int = None,
-        allow_redirects=True,
-    ) -> HttpTestResponse:
-        """
-        Send `PUT` request to `app` . See `spangle.testing.HttpTestClient.request` .
-        """
-        return self.request(
-            "put",
-            path,
-            params=params,
-            headers=headers,
-            cookies=cookies,
-            json=json,
-            files=files,
-            form=form,
-            content=content,
-            timeout=timeout,
-            allow_redirects=allow_redirects,
-        )
-
-    def patch(
-        self,
-        path: str,
-        params: Params = None,
-        headers: Headers = None,
-        cookies: Mapping = None,
-        json: Mapping = None,
-        files: Mapping = None,
-        form: Mapping = None,
-        content: bytes = None,
-        timeout: int = None,
-        allow_redirects=True,
-    ) -> HttpTestResponse:
-        """
-        Send `PATCH` request to `app` . See `spangle.testing.HttpTestClient.request` .
-        """
-        return self.request(
-            "patch",
-            path,
-            params=params,
-            headers=headers,
-            cookies=cookies,
-            json=json,
-            files=files,
-            form=form,
-            content=content,
-            timeout=timeout,
-            allow_redirects=allow_redirects,
-        )
-
-    def delete(
-        self,
-        path: str,
-        params: Params = None,
-        headers: Headers = None,
-        cookies: Mapping = None,
-        timeout: int = None,
-        allow_redirects=True,
-    ) -> HttpTestResponse:
-        """
-        Send `DELETE` request to `app` . See `spangle.testing.HttpTestClient.request` .
-        """
-        return self.request(
-            "delete",
-            path,
-            params=params,
-            headers=headers,
-            cookies=cookies,
-            timeout=timeout,
-            allow_redirects=allow_redirects,
-        )
